Function Append-Item($item) {
    if (Contains $script:items $item) { throw "duplicate definition of $($item.category) id: ${($item.id)}"}
    $script:items = $script:items + $item
}

Function Parse-Depends($dependsStringOrArray) {
    if (-not $dependsStringOrArray) { return @() }

    if ($dependsStringOrArray.GetType().Name -eq "String") {
        $matched = "$dependsStringOrArray" -match "(pack|file|command|service|exec|feature)\(([\w\.\-\:\\\ \(\)]+)\)"
        if (-not $matched) { throw }
        return @{category=$matches[1]; id=$matches[2]}
    }
    $dependsStringOrArray | % { Parse-Depends $_ }  
}


Function Check-Id($id, $category) {
    if (-not $id) { throw "Missing id for one $category." }
}

Function Get-ValueWithDefaultValue($config, $key, $defaultValue) {
    if ($config[$key]) { return $config[$key] } 
    else { return $defaultValue }
}

Function ConvertItem-FromConfig($config, $category) {
    $item = @{category = $category}
    Check-Id $config["id"] $category

    $config.keys | % {
        if ($_ -eq "depends") {
            $item["depends"] = (Parse-Depends $config["depends"])
	}
        else {
            $item[$_] = $config[$_]
	}               
    }
    return $item
}
