$scriptPath = Split-Path -parent $MyInvocation.MyCommand.Definition

$script:items = @()

. $scriptPath\common.ps1
. $scriptPath\file.ps1
. $scriptPath\pack.ps1
. $scriptPath\feature.ps1
. $scriptPath\exec.ps1
. $scriptPath\service.ps1

Function Generate-ExecutionPlan($items) {
    $ep = New-Object System.Collections.Generic.List``1[System.Object]
    $iteratePath = New-Object System.Collections.Generic.List``1[System.Object]
    Generate-ExecutionPlanRecur $items $items $ep $iteratePath
    return $ep
}

Function Generate-ExecutionPlanRecur($items, $allItems, $ep, $iteratePath) {
   $items | % {
       $item = $_
       Check-CycleDepend $iteratePath $item
       $iteratePath.Add($item)
       if ($item.depends) {
           $item.depends | ? { Depend-IsDefined $allItems $_ } | % { Generate-ExecutionPlanRecur  (MapTo-Item $allItems $_) $allItems  $ep $iteratePath}
       }
       $iteratePath.RemoveAt(($iteratePath.Count - 1))  
       if (-not (Contains $ep $item)) { $ep.Add($item) }
   }
}

Function MapTo-Item($items, $index) {
    $items | ? { ($_.id -eq $index.id) -and ($_.category -eq $index.category)}
}

Function Check-CycleDepend($iteratePath, $item) {
    $cycleDepends = Contains $iteratePath $item
    if ($cycleDepends) { throw "cycle depend found" } 
}

Function Depend-IsDefined($items, $item) {
    $contained = Contains $items $item
    if ($contained) { return $true }
    throw "{ $($item.category) $({$item.id}) } not defined."
}

Function Contains($items, $item) {
    $founded =  $items | ? { $_.id -eq $item.id } | ? { $_.category -eq $item.category }
    return ($founded -ne $nil)
}

Function Apply-Spec($executionPlan) {
    Write-Host "Applying spec"
    $executionPlan | % { & "Apply-$($_.category)" $_ } 
}
